/*
===================
JSLINT: set options
===================
*/

/*jshint esversion: 6*/
/*jshint sub:true*/

// Needs oolite >=1.82

/*
======
HEADER
======
*/

this.name						=	"harderHermits";
this.author						=	"UK_Eliter";
this.copyright 					=	"";
this.description				=	"harderHermits.js";
this.license					=	"2014 CC-by-nc-sa 3.0";


/*
==================
DEBUGGING SWITCHES
==================
*/

// this.logging = true;
var consoleDebugMessages 		=	false; // do NOT comment this out. Rather, to disable, set to false.
this.debug 						=	false;


/*
========================
JSLINT: start of wrapper
========================
*/

( function () {

"use strict";

/*
=======
GLOBALS
=======
*/

this.debug						=	false;

this.name						=	"Harder Hermits"

this.abandoned_chanceAct 		=	0.15; 	// 0.15
this.abandoned_quasi_chanceAct 	=	0.9; 	// 0.9.

this.hermit_chanceMoreEnergy	=	0.8; 	// 0.8.

this.installed_spicyHermits		=	false;	// gets changed to true if the OXP is installed.


/*
================
DEBUGGING
================
*/

this.$msg_dbg = function ( debugTxt )
{
	if ( !debugTxt ) { player.consoleMessage ( "Problem with debug message text!", 30 ); return; }
	player.consoleMessage ( this.name + " - " + debugTxt, 12);
	log ( this.name + " - DEBUG: ", debugTxt);
};

this.$msg_info = function ( msgTxt )
{
	if ( !msgTxt ) { player.consoleMessage ( "Problem with info message text!", 30 ); return; }
	log ( this.name + " - INFO: ", msgTxt);
};


/*
================
ENTITY PICKER(S)
================
*/

this.$isHermit = function ( e )
{
	return e.isShip && ( e.primaryRole === "rockhermit" || e.primaryRole === "rockhermit-chaotic" || e.primaryRole === "rockhermit-pirate" );
};

this.$isHermit_abandoned = function ( e )
{
	return e.isShip && e.primaryRole === "asteroid" && e.name === "Abandoned Rock Hermit";
	// Have to use name, because SpicyHermits changes the role, on spawning, to 'asteroid'.
};

this.$isHermit_abandonedQuasi = function ( e )
{
	return e.isShip && e.primaryRole === "spicy_hermits_abandoned" && e.hasRole ( "harderHermits_abandonedQuasi" )
};


/*
======
TIMERS
======
*/

this.$timer_delete_abandonedHermitAction_warnOnly = function ()
{
	if ( this.timer_abandonedHermitAction_warnOnly ) {
		if ( this.timer_abandonedHermitAction_warnOnly.isRunning ) {
			this.timer_abandonedHermitAction_warnOnly.stop ();
		} delete this.timer_abandonedHermitAction_warnOnly;
	}
};

this.$timer_delete_abandonedHermitAction_warnThenExplode = function ()
{
	if ( this.timer_abandonedHermitAction_warnThenExplode ) {
		if ( this.timer_abandonedHermitAction_warnThenExplode.isRunning ) {
			this.timer_abandonedHermitAction_warnThenExplode.stop ();
		} delete this.timer_abandonedHermitAction_warnThenExplode;
	}
};

this.$timer_delete_abandonedHermitAction_explode = function ()
{
	if ( this.timer_abandonedHermitAction_explode ) {
		if ( this.timer_abandonedHermitAction_explode.isRunning ) {
			this.timer_abandonedHermitAction_explode.stop ();
		} delete this.timer_abandonedHermitAction_explode;
	}
};

this.$timer_delete_all = function ()
{
	this.$timer_delete_abandonedHermitAction_warnOnly ();
	this.$timer_delete_abandonedHermitAction_warnThenExplode ();
	this.$timer_delete_abandonedHermitAction_explode ();
	delete this.abandonedHermit;
};


/*
===============
TIMED FUNCTIONS
===============
*/

this.$timed_abandonedHermitAction_message = function ( e )
{
	if ( ! e || ! e.isValid ) { return ; }
	e.commsMessage ( "WARNING: This station will explode. Clear the area." );
	e.broadcastCascadeImminent ();
};

this.$timed_abandonedHermitAction_warnOnly = function ()
{
	if ( this.debug ) { this.$msg_dbg ( "timed_abandonedHermitAction_warnOnly." ); }
	if ( ! this.abandonedHermit || ! this.abandonedHermit.isValid ) { return ; }
	var e = this.abandonedHermit;
	this.$timed_abandonedHermitAction_message ( e );
};

this.$timed_abandonedHermitAction_warnThenExplode = function ()
{
	if ( this.debug ) { this.$msg_dbg ( "timed_abandonedHermitAction_warnThenExplode." ); }
	this.$timer_delete_abandonedHermitAction_warnThenExplode ();
	if ( ! this.abandonedHermit || ! this.abandonedHermit.isValid ) { return ; }
	var e = this.abandonedHermit;
	this.$timed_abandonedHermitAction_message ( e );
	var seconds = Math.round ( Math.random () * 60 ); // * 60
	this.timer_abandonedHermitAction_explode = new Timer ( this, this.$timed_abandonedHermitAction_explode, seconds );
	this.$msg_dbg ( "Set timer for explosion at " + seconds + " seconds." );
};

this.$timed_abandonedHermitAction_explode = function ()
{
	if ( this.debug ) { this.$msg_dbg ( "timed_abandonedHermitAction_explode." ); }
	this.$timer_delete_abandonedHermitAction_explode ();
	if ( ! this.abandonedHermit || ! this.abandonedHermit.isValid ) { return ; }
	this.abandonedHermit.explode ();
	// NB SpicyHermits occasionally adds a q-mine death action!
	delete this.abandonedHermit;
};


/*
==============
EVENT HANDLERS
==============
*/

this.startUp = function ()
{
	if ( worldScripts [ "spicy_hermits_abandoned" ] ) {
		this.installed_spicyHermits = true;
		this.$msg_info ( "detected SpicyHermits OXP.");
	}
};

this.playerWillEnterWitchspace = function()
{
	this.$timer_delete_all ();
};

this.shipExitedWitchspace = function ()
{
	if ( system.isInterstellarSpace ) { return; }
	// log("Harder Hermits", "DEBUG: #1.");

	var l;
	var s;

	// TREAT NORMAL HERMITS

	s = system.filteredEntities ( this, this.$isHermit );
	l = s.length;
	if ( l === 0 ) { return; }

	var i;
	var oldMaxEnergy;
	var r;
	var str;
	// SpicyHermits gives pirate rock-hermits 2500 maxEnergy, and leaves the maxEnergy of other types alone, at 1,000.
	// SpicyHermits saves the position of its hermits but anything else - including their maxEnergy - about them.
	// We will not try to save that, either.
	if ( this.installed_spicyHermits ) {
		i = l;
		while ( i --) {
			if ( s[i] && s[i].isValid ) {
				r = Math.random ();
				if ( r < this.hermit_chanceMoreEnergy ) {
					r = Math.round ( r * 10 ) / 10;
					if ( s[i].primaryRole !== "rockhermit-pirate" ) {
						r += 0.6;
					} else {
						if ( r > 0.3 ) { r -= 0.1; }
					}
					// Tend to increase energy of the *non-pirates* more.
					oldMaxEnergy = s[i].maxEnergy
					s[i].maxEnergy *= ( 1 + r );
					// this.$msg_info ( "Harder Hermits", "FOR INFO: Increased the energy of a rock hermit." );
					str =  "FOR INFO: Increased the energy of rock hermit: " + s[i].displayName + " from " + oldMaxEnergy + " to " + s[i].maxEnergy;
					if ( this.debug ) { this.$msg_dbg ( str ); }
				}
			}
		}
	} else {
		while (i--) {
			if ( s[i] && s[i].isValid ) {
				r = Math.random ();
				if ( r < this.hermit_chanceMoreEnergy ) {
					r = Math.round ( r * 10 ) / 10;
					oldMaxEnergy = s[i].maxEnergy
					s[i].maxEnergy *= ( 1 + r );
					// this.$msg_info ( "Harder Hermits", "FOR INFO: Increased the energy of a rock hermit." );
					str = "FOR INFO: Increased the energy of rock hermit: " + s[i].displayName + " from " + oldMaxEnergy + " to " + s[i].maxEnergy;
					if ( this.debug ) { this.$msg_dbg ( str ); }
				}
			}
		}
	}

	if ( ! this.installed_spicyHermits ) { return; }

	// ABANDONED HERMITS (the fully abandoned ones)

	r = Math.random ();
	if ( r > this.abandoned_chanceAct ) { return ; }

	s = system.filteredEntities ( this, this.$isHermit_abandoned );
	l = s.length;
	if ( l > 0 ) {

		if ( this.debug ) { this.$msg_dbg ( "Sought and found abandoned hermits." ); }
		
		// pick one of them.
		var i = Math.floor ( Math.random () * l );

		if ( s[i] && s[i].isValid ) {
			// Irritatingly, cannot pass an argument to a timed function.
			this.abandonedHermit = s[i];
			if ( this.debug ) { this.abandonedHermit.displayName = "TESTING TESTING"; }
			var seconds = Math.ceil ( Math.random () * 10800 ) + 5; // * 10800 + 5
			var r = Math.random ();
			if ( r < 0.15 ) {
				this.timer_warnOnly = new Timer ( this, this.$timed_abandonedHermitAction_warnOnly, seconds );
				if ( this.debug ) { this.$msg_dbg ( "Setting to warn only." ); }
			} else if ( r < 0.25 ) {
				this.timer_blow = new Timer ( this, this.$timed_abandonedHermitAction_explode, seconds );
				if ( this.debug ) { this.$msg_dbg ( "Setting to explode only." ); }
			} else {
				this.timer_warnThenExplode = new Timer ( this, this.$timed_abandonedHermitAction_warnThenExplode, seconds );
				if ( this.debug ) { this.$msg_dbg ( "Setting to warn-then-explode." ); }
			}
			// this.$msg_info ( "Primed an abandoned rock hermit." );
			// var str = "Primed hermit " + this.abandonedHermit.displayName + " for action in " + seconds + " seconds time.";
			// this.$msg_dbg ( str )
		}
	}


	// QUASI-ABANDONED HERMITS (consider modifying their energy; shipdata creates them).

	if ( this.debug ) { this.$msg_dbg ( "Quasi-abandoned hermits." ); }

	r = Math.random ();
	if ( r > this.abandoned_quasi_chanceAct ) {
		if ( this.debug ) { this.$msg_dbg ( "Chance prevented proceeding with quasi-abandoned hermits." ); }
		return ;
	}

	if ( this.debug ) { this.$msg_dbg ( "Seeking quasi-abandoned hermits." ); }

	s = system.filteredEntities ( this, this.$isHermit_abandonedQuasi );
	l = s.length;
	if ( l === 0 ) {
		if ( this.debug ) { this.$msg_dbg ( "Found no quasi-abandoned hermits." ); }
		return;
	}

	if ( this.debug ) { this.$msg_dbg ( "Found > 0 quasi-abandoned hermit(s)." ); }
	
	// Go through them.
	i = l;
	while ( i --) {
		if ( s[i] && s[i].isValid ) {
			s[i].primaryRole = "rockhermit-pirate";
			// To reset the following done by the Spicy Hermit worldscript: ship.primaryRole = "asteroid".
			// Unclear this is needed.
			// Cannot change role (as against primaryRole) because role is read-only.
			// NB on ship.AI: 'If the ship is using Javascript-based AI, this will always be "nullAI.plist"'.
			// At present these hermits will launch ships which will just sit there, for some reason.

			r = Math.round ( Math.random () * 3000 );
			oldMaxEnergy = s[i].maxEnergy
			s[i].maxEnergy += r;
			if ( this.debug	) {
				s[i].displayName = "TESTING TESTING QUASI";
				str = "FOR INFO: Set the energy of rock hermit: " + s[i].displayName + " from " + oldMaxEnergy + " to " + s[i].maxEnergy;
				this.$msg_dbg ( str );
			} else {
				this.$msg_info ( "Harder Hermits", "FOR INFO: Set the energy of a rock hermit." );
			}
		}
	}
};


/*
=======================
JSLINT: end of wrapper
=======================
*/

}).call ( this );


// EOF
